#!/usr/bin/env python

# Add an albumful of pictures to TeknoHog's minimal PHP gallery on unix

# Requires convert(1), a part of the ImageMagick package.

# TODO:
# Image info (from exif.pl) and comments? Do we really need them?
# title.jpeg for each album, shown on the mainpage?

import sys, os, re

USAGE = """
Usage: """ + sys.argv[0] + """ <album name> image1 image2 ...
The album name should not have any spaces (it will become a directory name).
You can use shell expansions for image names, e.g. img*.jpeg."""

##########################################################
# Settings
##########################################################

# geometry options for image conversion
sizing = {"small":"480x480>", "large":"1024x1024>"}

# Nice (inverse priority) level
nice = 10

# Number of convert processes to run in parallel
ncpus = 4

targettype = ".jpeg"

##########################################################
# Code
##########################################################

# the directory where this file (or its incarnation) lies
basedir = os.path.dirname(sys.argv[0])

# File type is determined by a sensible suffix
sufpattern = r'\.[A-Za-z0-9]+$'

if len(sys.argv) < 3:
    print USAGE
    sys.exit()

album = sys.argv[1]
images = sys.argv[2:]

albumdir = os.path.join(basedir, album)

os.nice(nice)

# mkdir album if it's not there already
if not os.path.isdir(albumdir):
    for size in sizing.keys():
        os.makedirs(os.path.join(albumdir, size))

n = 0
for file in images:
    image = os.path.basename(file)
    print "Processing: [" + `images.index(file)+1` + "/" + `len(images)` + "] "+ image
    for size in sizing.keys():
        targetfile = os.path.join(albumdir, size, image)

        # targetfile will always be of a fixed type; convert
        # determines the type from the suffix
        targetfile = re.sub(sufpattern, targettype, targetfile)

        # automatic renaming of clashing images
        i = 0
        while 1:
            if os.path.isfile(targetfile):
                i += 1
                targetfile = re.sub(sufpattern, "." + `i` +  r'\g<0>', targetfile)
            else:
                break

        # Replace shell commandline with the more robust command
        # call. This is good when the filename contains weird
        # characters, and more rigorous style overall :)
        convert_args = ["convert", file, "-resize", sizing[size], targetfile]

        # For every ncpus'th process, wait for completion, otherwise
        # run in the background. This way we run _roughly_ ncpus
        # processes in parallel. (Could also foreground the last
        # process, but bg processes may still remain, so not really
        # necessary.)
        if n % ncpus == ncpus - 1:
            wait = os.P_WAIT
        else:
            wait = os.P_NOWAIT
        n += 1

        os.spawnvp(wait, convert_args[0], convert_args)

print "\nConversion complete."
if not os.path.isfile(os.path.join(albumdir, "heading.html")):
    print "You can write a heading for the album into the file " + albumdir + "/heading.html."
